/*
 * Copyright 2002-2008 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.springframework.config.java.valuesource;

import static org.hamcrest.CoreMatchers.equalTo;
import static org.junit.Assert.assertThat;

import org.junit.Before;
import org.junit.Test;

/**
 * Unit tests for {@link SystemPropertiesValueResolver}.
 *
 * @author Chris Beams
 */
public class SystemPropertiesValueResolverTests {

    private SystemPropertiesValueResolver resolver;
    private String key = "key";

    @Before
    public void setUp() {
        resolver = new SystemPropertiesValueResolver();
    }

    @Test(expected = ValueResolutionException.class)
    public void resolveNonExistentProperty() {
        // key has not been set as a property -> will throw
        resolver.getString(key);
    }

    @Test
    public void resolveExistentProperty() {
        String expected = "bar";
        System.setProperty(key, expected);
        
        String actual = resolver.getString(key);
        assertThat(actual, equalTo(expected));
    }
}
